<?php /*

 ocPortal
 Copyright (c) ocProducts, 2004-2012

 See text/EN/licence.txt for full licencing information.


 NOTE TO PROGRAMMERS:
   Do not edit this file. If you need to make changes, save your changed file to the appropriate *_custom folder
   **** If you ignore this advice, then your website upgrades (e.g. for bug fixes) will likely kill your changes ****

*/

/**
 * @license		http://opensource.org/licenses/cpal_1.0 Common Public Attribution License
 * @copyright	ocProducts Ltd
 * @package		ocf_forum
 */

/**
 * Module page class.
 */
class Module_admin_ocf_history
{

	/**
	 * Standard modular info function.
	 *
	 * @return ?array	Map of module info (NULL: module is disabled).
	 */
	function info()
	{
		$info=array();
		$info['author']='Chris Graham';
		$info['organisation']='ocProducts';
		$info['hacked_by']=NULL;
		$info['hack_version']=NULL;
		$info['version']=2;
		$info['locked']=false;
		return $info;
	}

	/**
	 * Standard modular entry-point finder function.
	 *
	 * @return ?array	A map of entry points (type-code=>language-code) (NULL: disabled).
	 */
	function get_entry_points()
	{
		return array('misc'=>'POST_HISTORY');
	}

	/**
	 * Standard modular run function.
	 *
	 * @return tempcode	The result of execution.
	 */
	function run()
	{
		if (get_forum_type()!='ocf') warn_exit(do_lang_tempcode('NO_OCF')); else ocf_require_all_forum_stuff();
		require_css('ocf');

		$type=get_param('type','misc');

		if ($type=='misc') return $this->gui();
		if ($type=='restore') return $this->restore();
		if ($type=='revert') return $this->revert();
		if ($type=='delete') return $this->delete();

		return new ocp_tempcode();
	}

	/**
	 * The UI to show the edit/delete history of posts (exact history shown depending on GET parameters).
	 *
	 * @return tempcode		The UI
	 */
	function gui()
	{
		check_specific_permission('view_content_history');

		$member_id=get_param_integer('member_id',-1);
		$post_id=get_param_integer('post_id',-1);
		$topic_id=get_param_integer('topic_id',-1);

		$where=array();
		if ($member_id!=-1)
		{
			$where['h_owner_member_id']=$member_id;
			$title=get_screen_title('POST_HISTORY_MEMBER');
		}
		if ($post_id!=-1)
		{
			$where['h_post_id']=$post_id;
			$title=get_screen_title('POST_HISTORY_POST');
		}
		if ($topic_id!=-1)
		{
			$where['h_topic_id']=$topic_id;
			$title=get_screen_title('POST_HISTORY_TOPIC');
		}
		if (count($where)==0)
		{
			$where=NULL;
			$title=get_screen_title('POST_HISTORY');
		}

		require_code('templates_internalise_screen');
		$test_tpl=internalise_own_screen($title);
		if (is_object($test_tpl)) return $test_tpl;

		$start=get_param_integer('start',0);
		$max=get_param_integer('max',40);

		$max_rows=$GLOBALS['FORUM_DB']->query_value('f_post_history','COUNT(*)',$where);

		$posts=$GLOBALS['FORUM_DB']->query_select('f_post_history',array('*'),$where,'ORDER BY h_action_date_and_time DESC',$max,$start);
		$content=new ocp_tempcode();
		foreach ($posts as $post)
		{
			$create_date_and_time=get_timezoned_date($post['h_create_date_and_time']);
			$action_date_and_time=get_timezoned_date($post['h_action_date_and_time']);
			$owner_member=$GLOBALS['FORUM_DRIVER']->member_profile_hyperlink($post['h_owner_member_id']);
			$alterer_member=$GLOBALS['FORUM_DRIVER']->member_profile_hyperlink($post['h_alterer_member_id']);

			// Action/Link
			$topic_exists=$GLOBALS['FORUM_DB']->query_value_null_ok('f_topics','id',array('id'=>$post['h_topic_id']));
			if (!is_null($topic_exists))
			{
				if (($post['h_action']=='EDIT_POST'))
				{
					$relates_to=build_url(array('page'=>'topicview','type'=>'findpost','id'=>$post['h_post_id']),get_module_zone('topicview'));
					$relates_to->attach('#post_'.strval($post['h_post_id']));
					$relates_text=do_lang_tempcode('VIEW_POST');
					$relates_tooltip='post#'.strval($post['h_post_id']);
				}
				elseif (($post['h_action']=='DELETE_POST'))
				{
					$relates_to=build_url(array('page'=>'topicview','id'=>$post['h_topic_id']),get_module_zone('topicview'));
					$relates_text=do_lang_tempcode('VIEW_TOPIC');
					$relates_tooltip='topic#'.strval($post['h_topic_id']);
				}
				$link=hyperlink($relates_to,$relates_text,false,true,$relates_tooltip);
			} else $link=new ocp_tempcode();
			$action=do_lang($post['h_action']);

			// Buttons
			$buttons=new ocp_tempcode();
			if (has_specific_permission(get_member(),'delete_content_history')) // Delete permanently
			{
				$url=build_url(array('page'=>'_SELF','type'=>'delete','h_id'=>$post['id']),'_SELF',NULL,true);
				$buttons->attach(do_template('SCREEN_ITEM_BUTTON',array('_GUID'=>'11c9f9ef4a646493544cb29778134960','IMMEDIATE'=>true,'URL'=>$url,'IMG'=>'delete','TITLE'=>do_lang_tempcode('DELETE_HISTORY_POST'))));
			}
			if ((has_specific_permission(get_member(),'restore_content_history')) && (!is_null($topic_exists)) && ($post['h_action']=='DELETE_POST')) // Restore
			{
				$url=build_url(array('page'=>'_SELF','type'=>'restore','h_id'=>$post['id']),'_SELF',NULL,true);
				$buttons->attach(do_template('SCREEN_ITEM_BUTTON',array('_GUID'=>'49623e00065f488bb27097bb722232dc','IMMEDIATE'=>true,'URL'=>$url,'IMG'=>'restore','TITLE'=>do_lang_tempcode('RESTORE_HISTORY_POST'))));
			}
			if ((has_specific_permission(get_member(),'restore_content_history')) && (!is_null($topic_exists)) && ($post['h_action']=='EDIT_POST')) // Restore
			{
				$url=build_url(array('page'=>'_SELF','type'=>'revert','h_id'=>$post['id']),'_SELF',NULL,true);
				$buttons->attach(do_template('SCREEN_ITEM_BUTTON',array('_GUID'=>'3f41d4d399676972c01ebb14f6ee56db','IMMEDIATE'=>true,'URL'=>$url,'IMG'=>'choose','TITLE'=>do_lang_tempcode('REVERT_HISTORY_POST'))));
			}

			$content->attach(do_template('OCF_HISTORY_POST',array('_GUID'=>'f3512689a8b3fcf4215f63f9f340cdac','LABEL'=>do_lang_tempcode('BEFORE_ACTION'),'LINK'=>$link,'BUTTONS'=>$buttons,'ACTION'=>$action,'ACTION_DATE_AND_TIME'=>$action_date_and_time,'ACTION_DATE_AND_TIME_RAW'=>strval($post['h_action_date_and_time']),'CREATE_DATE_AND_TIME_RAW'=>strval($post['h_create_date_and_time']),'CREATE_DATE_AND_TIME'=>$create_date_and_time,'OWNER_MEMBER'=>$owner_member,'ALTERER_MEMBER'=>$alterer_member,'BEFORE'=>$post['h_before'])));
		}
		if ((count($posts)!=0) && ($post_id!=-1))
		{
			$original_post=$GLOBALS['FORUM_DB']->query_select('f_posts',array('*'),array('id'=>$post_id),'',1);
			if (array_key_exists(0,$original_post))
			{
				$action=do_lang('CURRENT');
				$link=hyperlink(build_url(array('page'=>'topicview','type'=>'findpost','id'=>$post_id),get_module_zone('topicview')),do_lang_tempcode('VIEW_POST'),false,true);
				$buttons=new ocp_tempcode();
				$action_date_and_time='';
				$action_date_and_time_raw='';
				$owner_member=$GLOBALS['FORUM_DRIVER']->member_profile_hyperlink($original_post[0]['p_poster']);
				$alterer_member=new ocp_tempcode();
				$before=get_translated_text($original_post[0]['p_post'],$GLOBALS['FORUM_DB']);
				$create_date_and_time=get_timezoned_date($original_post[0]['p_time']);
				$create_date_and_time_raw=strval($original_post[0]['p_time']);
				$content2=do_template('OCF_HISTORY_POST',array('_GUID'=>'a3512689a8b3fcf4215f63f9f340cdac','LABEL'=>do_lang_tempcode('CURRENT_STATUS'),'LINK'=>$link,'BUTTONS'=>$buttons,'ACTION'=>$action,'ACTION_DATE_AND_TIME'=>$action_date_and_time,'ACTION_DATE_AND_TIME_RAW'=>$action_date_and_time_raw,'CREATE_DATE_AND_TIME_RAW'=>$create_date_and_time_raw,'CREATE_DATE_AND_TIME'=>$create_date_and_time,'OWNER_MEMBER'=>$owner_member,'ALTERER_MEMBER'=>$alterer_member,'BEFORE'=>$before));
				$content2->attach($content);
				$content=$content2;
			}
		}

		require_code('templates_pagination');
		$pagination=pagination(do_lang_tempcode('POST_HISTORY'),NULL,$start,'start',$max,'max',$max_rows,NULL,'misc',true);

		return do_template('OCF_HISTORY_SCREEN',array('_GUID'=>'7dd45ce985fc7222771368336c3f19e4','PAGINATION'=>$pagination,'TITLE'=>$title,'CONTENT'=>$content));
	}

	/**
	 * The actualiser to restore a deleted post.
	 *
	 * @return tempcode		The UI
	 */
	function restore()
	{
		check_specific_permission('restore_content_history');

		$title=get_screen_title('POST_HISTORY');

		$id=get_param_integer('h_id');
		$post=$GLOBALS['FORUM_DB']->query_select('f_post_history',array('*'),array('id'=>$id),'',1);
		if (!array_key_exists(0,$post)) warn_exit(do_lang_tempcode('MISSING_RESOURCE'));
		$post=$post[0];
		require_code('ocf_posts_action');
		require_code('ocf_posts_action2');
		$owner_username=$GLOBALS['FORUM_DRIVER']->get_username($post['h_owner_member_id']);
		if (is_null($owner_username)) $owner_username=do_lang('UNKNOWN');
		ocf_make_post($post['h_topic_id'],'',$post['h_before'],0,false,1,0,$owner_username,NULL,$post['h_create_date_and_time'],$post['h_owner_member_id'],NULL,time(),get_member(),false,true,NULL,true,'',0,NULL,false,false,false,false,NULL,false);

		$url=build_url(array('page'=>'_SELF','type'=>'misc'),'_SELF',NULL,true);
		return redirect_screen($title,$url,do_lang_tempcode('SUCCESS'));
	}

	/**
	 * The actualiser to revert an edited post.
	 *
	 * @return tempcode		The UI
	 */
	function revert()
	{
		check_specific_permission('restore_content_history');

		$title=get_screen_title('POST_HISTORY');

		$id=get_param_integer('h_id');
		$post=$GLOBALS['FORUM_DB']->query_select('f_post_history',array('*'),array('id'=>$id),'',1);
		if (!array_key_exists(0,$post)) warn_exit(do_lang_tempcode('MISSING_RESOURCE'));
		$post=$post[0];
		$post2=$GLOBALS['FORUM_DB']->query_select('f_posts',array('*'),array('id'=>$post['h_post_id']),'',1);
		if (!array_key_exists(0,$post2)) warn_exit(do_lang_tempcode('MISSING_RESOURCE'));
		$post2=$post2[0];
		require_code('ocf_posts_action');
		require_code('ocf_posts_action2');
		require_code('ocf_posts_action3');
		ocf_edit_post($post['h_post_id'],$post2['p_validated'],$post2['p_title'],$post['h_before'],0,$post2['p_is_emphasised'],$post2['p_intended_solely_for'],false,false,do_lang('REVERT_HISTORY_POST'));

		$url=build_url(array('page'=>'_SELF','type'=>'misc'),'_SELF',NULL,true);
		return redirect_screen($title,$url,do_lang_tempcode('SUCCESS'));
	}

	/**
	 * The actualiser to delete some element from the post history.
	 *
	 * @return tempcode		The UI
	 */
	function delete()
	{
		check_specific_permission('delete_content_history');

		$title=get_screen_title('POST_HISTORY');

		$GLOBALS['FORUM_DB']->query_delete('f_post_history',array('id'=>get_param_integer('h_id')),'',1);

		$url=build_url(array('page'=>'_SELF','type'=>'misc'),'_SELF',NULL,true);
		return redirect_screen($title,$url,do_lang_tempcode('SUCCESS'));
	}

}


